#!/usr/bin/python
# *********************************************************************
# * Copyright (C) 2012--2013 Luca Baldini (luca.baldini@pi.infn.it)   *
# *                                                                   *
# * For the license terms see the file LICENCE, distributed           *
# * along with this software.                                         *
# *********************************************************************
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
from plasduino.modulemanager.gModuleInfo import gModuleInfo

#-- PLASDUINO_MODULEINFO_BEGIN
__modulename__  = 'Pendulum'
__executable__  = os.path.basename(__file__)
__author__      = 'Luca Baldini'
__contact__     = 'luca.baldini@pi.infn.it'
__abstract__    = """Module for studying the period of a pendulum.

Based on a single photo-gate, it allows the measurement of the period of a
pendulum and the transit time at the bottom of the oscillation.
"""

__moduleinfo__ = gModuleInfo(__modulename__, __abstract__, __executable__,
                             author = __author__, contact = __contact__)
#-- PLASDUINO_MODULEINFO_END

import sys

from PyQt4 import QtGui
from plasduino.gui.gAcquisitionWindow import gAcquisitionWindow
from plasduino.daq.gArduinoRunControl import gArduinoRunControl
from plasduino.daq.gEventBuffer import gEventBuffer
from plasduino.__logging__ import logger



class pendulumEventBuffer(gEventBuffer):
    
    """ Specialized data buffer for the pendulum module.
    """

    PROCESSED_DATA_FIELDS = ['Time', 'Period', 'Transit time']
    PROCESSED_DATA_UNITS = ['s', 's', 's']
    MIN_NUM_EVENTS = 5

    def process(self, filePath):
        """ Process the data buffer.

        TODO: need to handle the case in which we start while the flag is
        in the middle of the optical gate. To all practical purposes this
        means we have to make sure the first timestamp is on the right edge
        (need to go in the lab and figure out which one that is).
        """
        logger.info('Writing processed data to %s...' % filePath)
        outputFile = open(filePath, 'w')
        header = self.getFileHeader()
        outputFile.writelines(header)
        t0 = self[0]['seconds']
        for i in xrange(3, len(self) - 1, 2):
            t = 0.5*(self[i]['seconds'] + self[i-1]['seconds']) - t0
            period = self[i+1]['seconds'] - self[i-3]['seconds']
            transit = self[i]['seconds'] - self[i-1]['seconds']
            outputFile.writelines('%.6f\t%.6f\t%.6f\n' % (t, period, transit))
        outputFile.close()
        logger.info('Done.')



class pendulumRunControl(gArduinoRunControl):

    SKETCH_NAME = 'timer_D2_change'

    """ Specialized RunControl class for the pendulum module.
    """

    def __init__(self, maxSeconds = None):
        """ Constructor.
        """
        eventBuffer = pendulumEventBuffer()
        gArduinoRunControl.__init__(self, eventBuffer, maxSeconds)

    def poll(self):
        """ Implementation of the polling cycle on the PC side.
        """
        gArduinoRunControl.poll_timestamps_alternating_edges(self)



class pendulumWindow(gAcquisitionWindow):

    """ Main window for the pendulum module.
    """

    WINDOW_TITLE = __modulename__
    MODULE_OPTIONS = ['gui.*', 'daq.*', 'arduino.*']

    def __init__(self, **kwargs):
        """ Constructor.
        """
        gAcquisitionWindow.__init__(self, **kwargs)
        self.RunControl = pendulumRunControl()
        self.connectRunControl()
        self.RunControl.connectToArduino(timeout = 0.2)



def launch():
    """ Launch the main application.
    """
    mainWindow = pendulumWindow()
    mainWindow.show()
    return mainWindow


if __name__ == '__main__':
    TOP_LEVEL_MODULE_OPTIONS = pendulumWindow.MODULE_OPTIONS
    from plasduino.__cfgparse__ import getapp
    application = getapp()
    mainWindow = launch()
    application.exec_() 
