#!/usr/bin/python
# *********************************************************************
# * Copyright (C) 2013 Luca Baldini (luca.baldini@pi.infn.it)         *
# *                                                                   *
# * For the license terms see the file LICENCE, distributed           *
# * along with this software.                                         *
# *********************************************************************
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
from plasduino.modulemanager.gModuleInfo import gModuleInfo

#-- PLASDUINO_MODULEINFO_BEGIN
__modulename__  = 'Pendulum View'
__executable__  = os.path.basename(__file__)
__author__      = 'Luca Baldini'
__contact__     = 'luca.baldini@pi.infn.it'
__abstract__    = """Track the motion of a pendulum continously in real time.

This application acquires data through the audiocard and needs a specific,
dedicated hardware setup.
"""

__moduleinfo__ = gModuleInfo(__modulename__, __abstract__, __executable__,
                             author = __author__, contact = __contact__)
#-- PLASDUINO_MODULEINFO_END


import sys
import time

from plasduino.__logging__ import logger
from plasduino.gui.gAnalogMonitorWindow import gAnalogMonitorWindow
from plasduino.gui.gStripChart import gStripChart
from plasduino.gui.gTransportBar import gTransportBar
from plasduino.gui.gQt import connect, disconnect
from plasduino.daq.gEventBuffer import gEventBuffer
from plasduino.daq.gAudioStereoEvent import gAudioStereoEvent
from plasduino.daq.gAudioStereoEventBuffer import gAudioStereoEventBuffer
from plasduino.daq.gAudioStereoRunControl import gAudioStereoRunControl
from PyQt4 import Qt, QtCore, QtGui


""" Brief description of the experimental setup.

We track the position of the pendulum through the PC audiocard by measuring
the resistance between the pendulum tip in water and an electrode.
There are two pendulums, connected with the L and R channels of the audiocard.

The output signal is modulated with a square wave with period ~2 ms and ~50%
duty cycle in order to be able to bypass the audiocard input capacitor
(we're interested in frequencies of the order of ~1 Hz or less).
"""


class pendulumviewEventBuffer(gAudioStereoEventBuffer):

    def __init__(self, sampleRate, offsetRight = 10000, offsetLeft = 10000,
                 maxNumEvents = None):
        """ Overloaded constructor.
        """
        gAudioStereoEventBuffer.__init__(self, sampleRate, 1, maxNumEvents)
        self.OffsetRight = offsetRight
        self.OffsetLeft = offsetLeft

    def setOffsets(self, offsetRight, offsetLeft):
        """ Set the offset for the data buffer.
        """
        self.OffsetRight = offsetRight
        self.OffsetLeft = offsetLeft

    def fill(self, data):
        """ Overloaded fill() method.
        """
        startIndex = len(self)
        right = 0
        left = 0
        numSamples = 0
        for index in xrange(0, len(data), self.EVENT_LENGTH):
            event = gAudioStereoEvent(data[index:index + self.EVENT_LENGTH])
            right += abs(event['right'])
            left += abs(event['left'])
            numSamples += 1
            self.CurrentIndex += 1
        if numSamples:
            event = {'seconds': self.TimeStep*(self.CurrentIndex-numSamples/2),
                     'right': right/float(numSamples) - self.OffsetRight,
                     'left': left/float(numSamples) - self.OffsetLeft}
            gEventBuffer.fill(self, event)

    def average(self):
        """ Return the average values for the right and left channels across
        the entire buffer.
        """
        logger.info('Calculating average across data buffer...')
        right = 0
        left = 0
        for event in self:
            right += event['right'] 
            left += event['left']
        if len(self):
            right /= float(len(self))
            left /= float(len(self))
        logger.info('Done. R channel: %.2f, L channel: %.2f.' % (right, left))
        return (right, left)

    def zeroOffset(self):
        """
        """
        (right, left) = self.average()
        self.setOffsets(right, left)



class pendulumviewRunControl(gAudioStereoRunControl):

    """ Specialized RunControl class for the pendulumview module.
    """

    def __init__(self, maxSeconds = None):
        """ Constructor.
        """
        from plasduino.__cfgparse__ import TOP_LEVEL_CONFIGURATION
        self.SampleRate = 8000
        self.BufferSize = 400
        eventBuffer = pendulumviewEventBuffer(self.SampleRate)
        gAudioStereoRunControl.__init__(self, eventBuffer, maxSeconds)

    def poll(self):
        """ Implementation of the polling cycle on the PC side.
        """
        self.Mutex.lock()
        try:
            data = self.AudioStream.read(self.BufferSize)
        except IOError:
            return
        except AttributeError:
            return
        self.Mutex.unlock()
        self.EventBuffer.fill(data)
        self.emit(QtCore.SIGNAL('readout(int)'), len(self.EventBuffer) - 1)

    def zeroOffsetInit(self, acquisitionSeconds = 3):
        """ Calibrate the offsets for the right and left channels.
        """
        logger.info('Starting zero-offset calibration...')
        self.EventBuffer.setOffsets(0, 0)
        timer = QtCore.QTimer(self)
        timer.setInterval(acquisitionSeconds*1000)
        timer.setSingleShot(True)
        connect(timer, 'timeout()', self.zeroOffsetDone)
        timer.start()
        self.setRunning()
        
    def zeroOffsetDone(self):
        """ Finalize the zero offset procedure.
        """
        self.setStopped()
        self.EventBuffer.zeroOffset()
        logger.info('Zero-offset calibration correctly performed.')



class pendulumviewWindow(gAnalogMonitorWindow):

    """ Main window for the pendulumview module.
    """

    WINDOW_TITLE = __modulename__
    MODULE_OPTIONS = ['gui.style', 'gui.skin', 'gui.language', 'daq.*']

    def __init__(self, **kwargs):
        """ Constructor.
        """
        gAnalogMonitorWindow.__init__(self, **kwargs)
        self.RunControl = pendulumviewRunControl()
        self.connectRunControl()
        self.insertMenu('Calibration', 'Configuration')
        self.addMenuAction('Calibration', 'Zero offset',
                           self.RunControl.zeroOffsetInit)
        self.PlotWidget.setYTitle('Analog input [a. u.]')
        kwargs = {'markerSize': 0, 'lineWidth': 2}
        self.addPlot('Right channel', color = Qt.Qt.red, **kwargs)
        self.addPlot('Left channel', color = Qt.Qt.blue, **kwargs)
        self.setupConnections()

    def setupConnections(self):
        """ Setup the connections.
        """
        connect(self.RunControl, 'started()', self.clearPlots)
        connect(self.RunControl, 'readout(int)', self.updatePlots)
        connect(self.RunControl, 'zeroOffsetInit()', self.setTransBarDisabled)
        connect(self.RunControl, 'zeroOffsetDone()', self.setTransBarStopped)
        connect(self.RunControl, 'started()', self.disableZooming)
        connect(self.RunControl, 'stopped()', self.enableZooming)

    def setTransBarDisabled(self):
        """ Disable the transport bar.
        """
        self.TransportBar.setStatus(gTransportBar.STATUS_DISABLED)

    def setTransBarStopped(self):
        """ Set the transport bar in the stopped state.
        """
        self.TransportBar.setStatus(gTransportBar.STATUS_STOPPED)

    def updatePlots(self, index):
        """ Synchronous update of the data display.
        """
        plotRight = self.getPlotByIndex(0)
        plotLeft = self.getPlotByIndex(1)
        evt = self.RunControl.EventBuffer[index]
        plotRight.addDataPoint(evt['seconds'], evt['right'])
        plotLeft.addDataPoint(evt['seconds'], evt['left'])
        self.PlotWidget.replot()


def launch():
    """ Launch the main application.
    """
    mainWindow = pendulumviewWindow()
    mainWindow.show()
    return mainWindow


if __name__ == '__main__':
    TOP_LEVEL_MODULE_OPTIONS = pendulumviewWindow.MODULE_OPTIONS
    from plasduino.__cfgparse__ import getapp
    application = getapp()
    mainWindow = launch()
    application.exec_() 
