#!/usr/bin/python
# *********************************************************************
# * Copyright (C) 2012--2013 Luca Baldini (luca.baldini@pi.infn.it)   *
# *                                                                   *
# * For the license terms see the file LICENCE, distributed           *
# * along with this software.                                         *
# *********************************************************************
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import math

from plasduino.modulemanager.gModuleInfo import gModuleInfo

#-- PLASDUINO_MODULEINFO_BEGIN
__modulename__ = 'Video Click'
__executable__  = os.path.basename(__file__)
__author__      = 'Luca Baldini'
__contact__     = 'luca.baldini@pi.infn.it'
__abstract__    = """Standalone module for analyzing video clips.

Fully-fledged module for opening video clips, exracting frames and tracking
the position of objects in the canvas by means of mouse clicks. The data can
be exported in plain text for offline analysis.
"""

__moduleinfo__ = gModuleInfo(__modulename__, __abstract__, __executable__, 
                             author = __author__, contact = __contact__)
#-- PLASDUINO_MODULEINFO_END


import time

from PyQt4 import QtGui, QtCore
from plasduino.gui.gVideoclickWindow import gVideoclickWindow
from plasduino.gui.gVideoclickZoomPixmapWidget import \
    gVideoclickZoomPixmapWidget
from plasduino.gui.gDataDisplay import gDataDisplay
from plasduino.gui.gQt import connect, disconnect
from plasduino.__logging__ import logger


class videoclickObjectPosition:

    """ Small utility class representing the position of an object within a
    pixmap.
    """
    
    def __init__(self, x, y):
        """ Constructor.
        """
        self.X = x
        self.Y = y

    def __mul__(self, other):
        """ Multiply the coordinates by a scale factor.
        """
        return videoclickObjectPosition(other*self.X, other*self.Y)
    
    def __add__(self, other):
        """ Add points.
        """
        return videoclickObjectPosition(self.X + other.X, self.Y + other.Y)

    def __sub__(self, other):
        """ Subtract points.
        """
        return videoclickObjectPosition(self.X - other.X, self.Y - other.Y)

    def __str__(self):
        """ String formatting.
        """
        return '(%d, %d)' % (self.X, self.Y)



def dist(p1, p2):
    """ Retun the distance between two points.
    """
    return math.sqrt((p1.X - p2.X)**2 + (p1.Y - p2.Y)**2)



class videoclickEventBuffer:
    
    """ Event buffer fot the videpoint module.

    Needless to say, this has nothing to do with the event buffers for the
    other modules, except for the fact that we didn't bother searching for
    a different name.

    This module is peculiar in that frames can be clicked in random order and
    can be overwritten one or more time. This would suggest some sparse data
    structure like a dictionary. Still, in the end we want to save the
    data preserving the frame ordering, so we go for a list (a list of lists,
    actually) instead. The event list is filled at the beginning with a
    bunch of None which get overwritten as the user clicks on the frame.
    This is highly inefficient, memor-wise, but with the numbers we're
    dealing with nobody will ever notice it.

    Note that the frames and the objects are numbered starting from 1
    rathen than 0, so we need to take this into account in the get/set methods.
    """

    def __init__(self, numFrames, numObjects, frameRate):
        """ Constructor.
        """
        self.NumFrames = numFrames
        self.NumObjects = numObjects
        self.FrameRate = frameRate
        self.RefSystemScale = None
        self.RefSystemOrigin = videoclickObjectPosition(0, 0)
        self.clear()

    def empty(self):
        """ Return True if the buffer is empty (i.e. no position has been
        added, yet).
        """
        return self.__Empty

    def clear(self):
        """ Clear the data buffer (i.e., set all the positions to None).
        """
        logger.info('Clearing up the videoclick event buffer...')
        self.__TimestampList = []
        self.__PositionList = []
        for frameNumber in range(self.NumFrames):
            self.__TimestampList.append(frameNumber/float(self.FrameRate))
            self.__PositionList.append([None]*self.NumObjects)
        self.__Empty = True
        logger.info('Done.')

    def clearFrame(self, frameNumber):
        """ Clear a frame (i.e., set all the positions to None).
        """
        logger.info('Clearing frame %s...' % frameNumber)
        self.__PositionList[frameNumber - 1] = [None]*self.NumObjects

    def getTimestamp(self, frameNumber):
        """ Return the timestamp for a given frame.
        """
        return self.__TimestampList[frameNumber - 1]

    def getPosition(self, frameNumber, objectNumber):
        """ Return the position of a given object in a given frame.
        """
        return self.__PositionList[frameNumber - 1][objectNumber - 1]

    def setPosition(self, frameNumber, objectNumber, position):
        """ Set the position for a given object in a given frame.
        """
        self.__PositionList[frameNumber - 1][objectNumber - 1] = position
        if self.hasFrameData(frameNumber):
            self.__Empty = False

    def hasFrameData(self, frameNumber):
        """ Return true if the buffer contains valid data for all the objects in
        a given frame.
        """
        positions = [self.getPosition(frameNumber, objectNumber) for \
                         objectNumber in range(1, self.NumObjects + 1)]
        return not (None in positions)

    def getTxtFileHeader(self):
        """ Utility method to format the header for the data file.
        """
        timestamp = time.asctime()
        header =  '#\n'
        header += '# Written by plasduino.%s on %s.\n' %\
            (self.__class__.__name__, timestamp)
        header += '# Frame\tTime\t'
        for objectNumber in range(1, self.NumObjects + 1):
            header += 'x%d\ty%d\t' % (objectNumber, objectNumber)
        header = '%s\n' % header.rstrip('\t')
        header += '# []\t[s]\t'
        if self.RefSystemScale is None:
            for objectNumber in range(self.NumObjects):
                header += '[pixel]\t[pixel]\t'
        else:
            for objectNumber in range(self.NumObjects):
                header += '[cm]\t[cm]\t'
        header = '%s\n' % header.rstrip('\t')
        header += '#\n'
        return header

    def getCsvFileHeader(self):
        """ Utility method to format the header for the data file.
        """
        header = 'Frame,Time,'
        for objectNumber in range(1, self.NumObjects + 1):
            header += 'x%d,y%d,' % (objectNumber, objectNumber)
        header = '%s\r\n' % header.rstrip(',')
        header += '[],[s],'
        if self.RefSystemScale is None:
            for objectNumber in range(self.NumObjects):
                header += '[pixel],[pixel],'
        else:
            for objectNumber in range(self.NumObjects):
                header += '[cm],[cm],'
        header = '%s\r\n' % header.rstrip(',')
        return header

    def getOutputRow(self, frameNumber, separator = '\t', eol = '\n'):
        """ Return an output line of text (for a given frame number) to be
        written in the output file.

        We want to make sure that every single line in the output file has
        complete information, so that the plain text file can be processed
        by any common plotting package.
        """
        positions = [self.getPosition(frameNumber, objectNumber) for \
                         objectNumber in range(1, self.NumObjects + 1)]
        if None in positions:
            return None
        row = '%d%s%.3f%s' % (frameNumber, separator,
                              self.getTimestamp(frameNumber), separator)
        for position in positions:
            position -= self.RefSystemOrigin
            if self.RefSystemScale is None:
                row += '%d%s%d%s' % (position.X, separator, position.Y,
                                     separator)
            else:
                position *= self.RefSystemScale
                row += '%.2f%s%.2f%s' % (position.X, separator, position.Y,
                                         separator)
        row = '%s%s' % (row.rstrip(separator), eol)
        return row

    def writeTxt(self, filePath):
        """ Write the buffer to file in text format.
        """
        logger.info('Writing data to %s...' % filePath)
        outputFile = open(filePath, 'w')
        header = self.getTxtFileHeader()
        outputFile.writelines(header)
        for frameNumber in range(1, self.NumFrames + 1):
            row = self.getOutputRow(frameNumber)
            if row is not None:
                outputFile.writelines(row)
        outputFile.close()
        logger.info('Done.')

    def writeCsv(self, filePath):
        """ Write the buffer to file in text format.
        """
        logger.info('Writing data to %s...' % filePath)
        outputFile = open(filePath, 'w')
        header = self.getCsvFileHeader()
        outputFile.writelines(header)
        for frameNumber in range(1, self.NumFrames + 1):
            row = self.getOutputRow(frameNumber, ',', '\r\n')
            if row is not None:
                outputFile.writelines(row)
        outputFile.close()
        logger.info('Done.')



class videoclickDataDisplay(gDataDisplay):
    
    """ Custom data display for the videoclick module.
    """

    LABEL_WIDTH = 125
    DATA_WIDTH = 110
 
    def __init__(self, parent = None, numObjects = 2):
        """ Constructor.
        """
        gDataDisplay.__init__(self, parent, minLabelWidth = self.LABEL_WIDTH,
                              minDataWidth = self.DATA_WIDTH)
        self.addFrameNumberField()
        self.addElapsedTimeField()
        for objectNumber in range(1, numObjects + 1):
            self.addObjectPositionField(objectNumber)
        self.setFieldDataFormat(self.ELAPSED_TIME_FIELD_NAME, '%.2f')

    def sizeHint(self):
        """ Overloaded function for managing the size policy.
        """
        return QtCore.QSize(gVideoclickZoomPixmapWidget.DEFAULT_SIZE, 10)



class videoclickSystemDisplay(gDataDisplay):
   
    """ Custom data display for the videoclick coordinate system.

    We display here the relevant information about the origin of the
    coordinate system and the scale factor for converting between pixels
    and physical units.
    """

    def __init__(self, parent = None):
        """ Constructor.
        """
        gDataDisplay.__init__(self, parent, title = 'Reference system',
                              minLabelWidth = videoclickDataDisplay.LABEL_WIDTH,
                              minDataWidth = videoclickDataDisplay.DATA_WIDTH)
        self.addRefSystemScaleField()
        self.addRefSystemOriginField()

    def update(self, eventBuffer):
        """ Update the reference system information according to the
        setting of the input event buffer.
        """
        self.setRefSystemScale(eventBuffer.RefSystemScale)
        self.setRefSystemOrigin(eventBuffer.RefSystemOrigin)

    def sizeHint(self):
        """ Overloaded function for managing the size policy.
        """
        return QtCore.QSize(gVideoclickZoomPixmapWidget.DEFAULT_SIZE, 10)



class videoclickBaseDialog(QtGui.QDialog):

    """ Base class for the dialogs to set the origin and the scale factor
    for the reference system.
    """

    def __init__(self, parent, windowTitle, text):
        """ Constructor.
        """
        QtGui.QDialog.__init__(self, parent)
        self.setWindowTitle(windowTitle)
        self.setFixedWidth(400)
        self.Layout = QtGui.QGridLayout(self)
        self.Label = QtGui.QLabel()
        self.Label.setWordWrap(True)
        self.Label.setText(text)
        self.Layout.addWidget(self.Label, 0, 0, 1, 2)
        buttonBox = QtGui.QDialogButtonBox(QtGui.QDialogButtonBox.Save |
                                           QtGui.QDialogButtonBox.Cancel)
        self.SaveButton = buttonBox.button(QtGui.QDialogButtonBox.Save)
        self.SaveButton.setEnabled(False)
        connect(buttonBox, 'accepted()', self.accept)
        connect(buttonBox, 'rejected()', self.reject)
        self.Layout.addWidget(buttonBox, 2, 0, 1, 2)
        self.acceptMouseEvents()

    def acceptMouseEvents(self):
        """ Connect the appropriate slot to react to the mouse clicks on the
        parent pixmap.
        """
        connect(self.parent().PixmapWidget, 'frameLeftClicked(int, int)',
                self.leftClick)
        connect(self.parent().PixmapWidget, 'frameRightClicked(int, int)',
                self.rightClick)

    def ignoreMouseEvents(self):
        """ Disconnect from the parent pixmap.

        This should be called when the dialog is closed.
        """
        disconnect(self.parent().PixmapWidget, 'frameLeftClicked(int, int)',
                   self.leftClick)
        disconnect(self.parent().PixmapWidget, 'frameRightClicked(int, int)',
                   self.rightClick)

    def leftClick(self):
        """ Do-nothing slot.
        """
        pass

    def rightClick(self):
        """ Do-nothing slot.
        """
        pass

    def reject(self):
        """ Overloade reject() method
        """
        self.ignoreMouseEvents()
        self.parent().repaintFrame()
        QtGui.QDialog.reject(self)



class videoclickOriginDialog(videoclickBaseDialog):

    """ Small dialog window to set the origin of the coordinate system.
    """

    def __init__(self, parent = None):
        """ Constructor.
        """
        windowTitle = 'Set the origin of the coordinate system'
        text = 'Click with left button on the frame on the main window to ' \
            'mark the origin of the reference system, then push the save ' \
            'button to save your choice, cancel to ignore it.'
        videoclickBaseDialog.__init__(self, parent, windowTitle, text)
        self.RefSystemOrigin = None

    def leftClick(self, x, y):
        """ Slot for the left mouse click. 
        """
        self.RefSystemOrigin = videoclickObjectPosition(x, y)
        self.SaveButton.setEnabled(True)
        self.parent().repaintFrame()
        self.parent().drawPoint(self.RefSystemOrigin, 'Origin')

    def accept(self):
        """ Overloaded accept() method.
        """
        logger.info('Setting the origin of the reference system to %s.' %\
                        self.RefSystemOrigin)
        self.ignoreMouseEvents()
        self.parent().EventBuffer.RefSystemOrigin = self.RefSystemOrigin
        self.parent().repaintFrame()
        self.parent().updateSystemDisplay()
        QtGui.QDialog.accept(self)



class videoclickScaleDialog(videoclickBaseDialog):

    """ Small dialog window to set the origin of the coordinate system.
    """

    def __init__(self, parent = None):
        """ Constructor.
        """
        windowTitle = 'Set the scale factor for the coordinate system'
        text = 'Use the left and right mouse buttons to mark the two ends ' \
            'of a known lenght on the frame in the main window, fill in the ' \
            'value of the lenght in the text box, then push the save button ' \
            'to save your choice, cancel to ignore it.'
        videoclickBaseDialog.__init__(self, parent, windowTitle, text)
        self.Layout.addWidget(QtGui.QLabel('Known length [cm]', self), 1, 0)
        self.LineEdit = QtGui.QLineEdit(self)
        self.Layout.addWidget(self.LineEdit, 1, 1)
        self.Layout.setColumnMinimumWidth(0, self.width()/2 - 4)
        connect(self.LineEdit, 'textChanged(QString)', self.checkData)
        self.Point1 = None
        self.Point2 = None
        self.ScaleFactor = None

    def checkData(self):
        """ Check whether we have all the pieces in place to set the scale
        factor (and set the status of the save button accordingly).
        """
        try:
            scale = float(self.LineEdit.text())
        except ValueError:
            self.SaveButton.setEnabled(False)
            return
        if self.Point1 and self.Point2:
            d = dist(self.Point1, self.Point2)
            if d > 0 and scale > 0:
                self.SaveButton.setEnabled(True)
                self.ScaleFactor = scale/d
                return
            else:
                self.SaveButton.setEnabled(False)
        else:
            self.SaveButton.setEnabled(False)

    def leftClick(self, x, y):
        """ Slot for the left mouse click. 
        """
        self.Point1 = videoclickObjectPosition(x, y)
        self.parent().repaintFrame()
        self.parent().drawPoint(self.Point1, 'p1')
        if self.Point2 is not None:
            self.parent().drawPoint(self.Point2, 'p2')
            self.parent().drawLine(self.Point1, self.Point2)
            self.checkData()

    def rightClick(self, x, y):
        """ Slot for the right mouse click. 
        """
        self.Point2 = videoclickObjectPosition(x, y)
        self.parent().repaintFrame()
        self.parent().drawPoint(self.Point2, 'p2')
        if self.Point1 is not None:
            self.parent().drawPoint(self.Point1, 'p1')
            self.parent().drawLine(self.Point1, self.Point2)
            self.checkData()

    def accept(self):
        """ Overloaded accept() method.
        """
        logger.info('Setting the reference system scale factor to %s.' %\
                        self.ScaleFactor)
        self.ignoreMouseEvents()
        self.parent().EventBuffer.RefSystemScale = self.ScaleFactor
        self.parent().repaintFrame()
        self.parent().updateSystemDisplay()
        QtGui.QDialog.accept(self)



class videoclickWindow(gVideoclickWindow):
    
    """ Main window for the videoclick module.
    """

    WINDOW_TITLE = __modulename__
    MODULE_OPTIONS = ['gui.style', 'gui.skin', 'gui.language',
                      'daq.tmp-file-dir', 'video.*']

    def __init__(self, **kwargs):
        """ Constructor.
        """
        gVideoclickWindow.__init__(self, **kwargs)
        self.insertMenuAction('File', 'Save data to file', self.save, 2)
        self.getMenuAction('File', 'Save data to file').setEnabled(False)
        self.insertMenu('Reference system', 'Configuration')
        self.addMenuAction('Reference system', 'Set scale factor',
                           self.setRefSystemScale)
        self.addMenuAction('Reference system', 'Set origin',
                           self.setRefSystemOrigin)
        self.getMenu('Reference system').setEnabled(False)
        from plasduino.__cfgparse__ import TOP_LEVEL_CONFIGURATION
        self.NumObjects = TOP_LEVEL_CONFIGURATION['video.num-objects']
        self.DataDisplay = videoclickDataDisplay(self, self.NumObjects)
        self.addWidget(self.DataDisplay, 3, 2)
        self.SystemDisplay = videoclickSystemDisplay(self)
        self.addWidget(self.SystemDisplay, 4, 2)
        self.CurrentObject = None
        self.acceptMouseEvents()
        self.EventBuffer = None

    def acceptMouseEvents(self):
        """ Setup the signal/slot connections for processing mouse events
        from the main pixmap widget.
        """
        connect(self.PixmapWidget, 'frameLeftClicked(int, int)', self.leftClick)

    def ignoreMouseEvents(self):
        """ Disconnect from the mouse event signals from the main pixmap widget.
        """
        disconnect(self.PixmapWidget, 'frameLeftClicked(int, int)',
                   self.leftClick)

    def updateSystemDisplay(self):
        """ Update the system display according to the settings of the event
        buffer.
        """
        self.SystemDisplay.update(self.EventBuffer)

    def setRefSystemScale(self):
        """
        """
        self.ignoreMouseEvents()
        dialog = videoclickScaleDialog(self)
        connect(dialog, 'finished(int)', self.acceptMouseEvents)
        dialog.show()

    def setRefSystemOrigin(self):
        """
        """
        self.ignoreMouseEvents()
        dialog = videoclickOriginDialog(self)
        connect(dialog, 'finished(int)', self.acceptMouseEvents)
        dialog.show()

    def openClipFinalize(self):
        """ Overloaded method from the base class.

        Note that the base class always calls its own setFrame() method
        and, as the overloaded one involved the event buffer, we have
        to call again self.setFrame(1) after the buffer has been creted.
        """
        gVideoclickWindow.openClipFinalize(self)
        numFrames = self.VideoDecoder.getNumFrames()
        frameRate = self.VideoDecoder.FrameRate
        self.EventBuffer = videoclickEventBuffer(numFrames, self.NumObjects,
                                                 frameRate)
        self.updateSystemDisplay()
        self.setFrame(1)
        self.getMenu('Reference system').setEnabled(True)
        
    def closeClip(self):
        """ Overloaded method from the base class.
        """
        gVideoclickWindow.closeClip(self)
        self.EventBuffer = None
        self.getMenuAction('File', 'Save data to file').setEnabled(False)
        self.getMenu('Reference system').setEnabled(False)

    def save(self):
        """ Save the current data file.
        """
        from plasduino.gui.gFileDialog import getSaveFilePath, FILE_TYPE_TXT,\
            FILE_TYPE_CSV
        filePath = getSaveFilePath([FILE_TYPE_TXT, FILE_TYPE_CSV], self,
                                   'Save data')
        if filePath is not None:
            if filePath.endswith(FILE_TYPE_TXT.Extension):
                self.EventBuffer.writeTxt(filePath)
            elif filePath.endswith(FILE_TYPE_CSV.Extension):
                self.EventBuffer.writeCsv(filePath)

    def setFrame(self, frameNumber):
        """ Set the frame to be displayed.
        
        While all the logic related to the GUI is encapsulated in the base
        class, here we update in addition the stuff related to the
        actual data.
        """
        gVideoclickWindow.setFrame(self, frameNumber)
        self.CurrentObject = 1
        self.DataDisplay.setFrameNumber('%d/%d' % (frameNumber,
                                                   self.EventBuffer.NumFrames))
        timestamp = self.EventBuffer.getTimestamp(frameNumber)
        self.DataDisplay.setElapsedTime(timestamp)
        for objectNumber in range(1, self.NumObjects + 1):
            position = self.EventBuffer.getPosition(frameNumber, objectNumber)
            self.DataDisplay.setObjectPosition(objectNumber, position)
            if position is not None:
                self.drawPoint(position, 'Object %d' % objectNumber)

    def drawPoint(self, point, text = ''):
        """ Draw a point on the pixmap, possibly with a text label.
        """
        pixmap = self.PixmapWidget.pixmap()
        painter =  QtGui.QPainter(pixmap)
        painter.setPen(QtGui.QPen(QtCore.Qt.white, 2))
        painter.drawEllipse(point.X, point.Y, 2, 2)
        text = '%s %s' % (text, point)
        painter.drawText(point.X - 10, point.Y - 10, text.strip())
        del painter
        self.repaint()

    def drawLine(self, point1, point2):
        """ Draw a line connectring two points on the pixmap widget.
        """
        pixmap = self.PixmapWidget.pixmap()
        painter =  QtGui.QPainter(pixmap)
        painter.setPen(QtGui.QPen(QtCore.Qt.white, 1))
        painter.drawLine(point1.X, point1.Y, point2.X, point2.Y)
        del painter
        self.repaint()

    def leftClick(self, x, y):
        """ Slot connected to a left mouse click on the pixmap widget.

        This is actually one of the main methods of the derived class.
        When we click on the pixmap we want to:
        * retrieve the position of the mouse click;
        * display the position on the data display;
        * set the position in the proper object in the event buffer;
        * advance the current object and, if necessary, the current frame.

        Note that we do buffer in time the operations via a QTimer that
        is instantiated at each click and only lives in this scope (so that
        we don't have to bother disconnecting signals and slots).
        We disconnect the slots receiving the mouse events as soon as we enter
        the slot and we reconnect them at the timeout of the timer. This is
        to ensure a minimum time between two valid clicks and avoid
        double-clicks, that we don't want in this context.
        At the last object we connect the timer to the next() slot and set
        the timeout to a larger value so that the last position pertists for
        some time on the data display.
        """
        # Disconnect the mouse signals.
        self.ignoreMouseEvents()
        # Are we clicking on the first object of a frame for which the first
        # object has already a position set? If so: clear the frame in the
        # event buffer and call setFrame()---which repaints the frame and
        # set the data display values.
        if self.CurrentObject == 1 and \
                self.EventBuffer.getPosition(self.CurrentFrame, 1) is not None:
            self.EventBuffer.clearFrame(self.CurrentFrame)
            self.setFrame(self.CurrentFrame)
        # Setup the timer.
        timer = QtCore.QTimer(self)
        timer.setSingleShot(True)
        connect(timer, 'timeout()', self.acceptMouseEvents)
        self.showMessage('Left mouse click at (%d, %d)' % (x, y))
        # Grab the position, update the data display, update the event buffer
        # and draw the point.
        position = videoclickObjectPosition(x, y)
        self.DataDisplay.setObjectPosition(self.CurrentObject, position)
        self.EventBuffer.setPosition(self.CurrentFrame, self.CurrentObject,
                                     position)
        self.drawPoint(position, 'Object %d' % self.CurrentObject)
        # Do we have enough points for the File->Save action to make sense?
        if not self.getMenuAction('File', 'Save data to file').isEnabled() and \
                not self.EventBuffer.empty():
            self.getMenuAction('File', 'Save data to file').setEnabled(True)
        # Update the object counter and start the timer.
        if self.CurrentObject == self.NumObjects and \
                self.CurrentFrame != self.EventBuffer.NumFrames:
            self.CurrentObject = 1
            timer.setInterval(750)
            connect(timer, 'timeout()', self.next)
        else:
            self.CurrentObject += 1
            timer.setInterval(250)
        timer.start()



def launch():
    """ Launch the main application.
    """
    mainWindow = videoclickWindow()
    mainWindow.show()
    return mainWindow


if __name__ == '__main__':
    TOP_LEVEL_MODULE_OPTIONS = videoclickWindow.MODULE_OPTIONS
    from plasduino.__cfgparse__ import getapp
    application = getapp()
    mainWindow = launch()
    application.exec_() 
