#!/usr/bin/env python
# *********************************************************************
# * Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)         *
# *                                                                   *
# * For the license terms see the file LICENCE, distributed           *
# * along with this software.                                         *
# *********************************************************************
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import sys
import re
import os
import __arduino__
import plasduino.__utils__

from plasduino.__logging__ import logger
from plasduino.__plasduino__ import PLASDUINO_ARDUINO



class gSketchLoader:

    """ Small utility class to handle the uploading process for the arduino
    sketches.
    """

    def __init__(self, port, model):
        """ Constructor.
        """
        self.Port = port
        self.Model = model
        logger.info('Retrieving upload information...')
        self.UploadProtocol = __arduino__.getUploadProtocol(self.Model)
        if self.UploadProtocol == 'stk500':
            self.UploadProtocol = 'stk500v1'
            logger.info('Upload protocol changed from stk500 to stk500v1.')
        logger.info('Upload protocol: %s.' % self.UploadProtocol)
        self.UploadSpeed = __arduino__.getUploadSpeed(self.Model)
        logger.info('Upload speed: %s.' % self.UploadSpeed)
        self.MCU = __arduino__.getMCU(self.Model)
        logger.info('MCU: %s.' % self.MCU)
       
    def sendReset(self):
        """ Send a reset command.
        """
        __arduino__.pulseDTR(self.Port)
 
    def upload(self, sketchName):
        """ Upload a sketch on the board.
        
        Note that we actually:
        (*) send a reset signal on the serial port (i.e., pulse the DTR).
        (*) upload the sketch to the flash memory;
        (*) send another reset so that we're ready to read again.
        """
        logger.info('Preparing to upload the sketch "%s"...' % sketchName)
        targetDirPath = os.path.join(PLASDUINO_ARDUINO, sketchName)
        targetFileName = __arduino__.getHexFileName(sketchName, self.Model)
        targetFilePath = os.path.join(targetDirPath, targetFileName)
        if not os.path.exists(targetFilePath):
            sys.exit('Could not find file %s. Abort.' % targetFilePath)
        else:
            logger.info('File %s found.' % targetFilePath)
        cmd = 'cd %s' % targetDirPath
        cmd = '%s; avrdude -V -F -c %s -b %s -p %s -P %s' %\
            (cmd, self.UploadProtocol, self.UploadSpeed, self.MCU, self.Port)
        cmd = '%s -U flash:w:%s' % (cmd, targetFileName)
        self.sendReset()
        status = plasduino.__utils__.cmd(cmd)
        if status:
            logger.warn('Giving up on the last reset...')
            return status
        self.sendReset()
        return 0



if __name__ == '__main__':
    loader = gSketchLoader('/dev/ttyACM0', 'uno')
    loader.upload('timerinterrupt')
