#!/usr/bin/env python
# *********************************************************************
# * Copyright (C) 2012--2013 Luca Baldini (luca.baldini@pi.infn.it)   *
# *                                                                   *
# * For the license terms see the file LICENCE, distributed           *
# * along with this software.                                         *
# *********************************************************************
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import serial
import sys
import os
import time
import __usbid__

from plasduino.daq.gTimestamp import gTimestamp 
from plasduino.daq.gAnalogReadout import gAnalogReadout
from plasduino.__logging__ import logger, abort
from plasduino.arduino.__arduino__ import BAUD_RATE, EOL_CHAR,\
    getSketchVersion, pulseDTR



class gArduinoManager(serial.Serial):

    """ Class managing a device connected through the serial interface
    """

    ONE  = '\x01'
    ZERO = '\x00'
    COULD_NOT_AUTODETECT_USB_ERR = 'Could not autodetect USB port.'

    def __init__(self):
        """ Constructor.
        """
        self.SketchName = None
        self.SketchVersion = None
        self.__Connected = False
        self.__SerialLog = None
        self.Port, self.Model, self.VID, self.PID = __usbid__.getArduinoInfo()
        if self.Port is None:
            msg = self.COULD_NOT_AUTODETECT_USB_ERR
            logger.error(msg)
            self.__SerialLog = msg
            return 
        try:
            pulseDTR(self.Port)
            serial.Serial.__init__(self, self.Port, BAUD_RATE)
            self.__Connected = True
        except Exception as e:
            msg = 'Error while talking over the serial port ("%s").' % e
            logger.info(msg)
            self.__SerialLog = msg

    def getSerialLog(self):
        """ Return the last info/warning/error message from the serial port,
        if any.
        """
        return self.__SerialLog

    def connected(self):
        """ Return true if properly connected with the serial device.
        """
        return self.__Connected

    def open(self):
        """ Overloaded open() function.

        Here we are just logging an acknowledgment and then calling the base
        class method.
        """
        logger.info('Opening serial connection to %s (baudrate = %d)...' %\
                        (self.Port, BAUD_RATE))
        serial.Serial.open(self)
        self.__Connected = True

    def close(self):
        """ Overloaded close() function.

        Here we are just logging an acknowledgment and then calling the base
        class method.
        """
        if self.__Connected:
            logger.info('Closing serial connection to %s...' % self.Port)
            serial.Serial.close(self)
        self.__Connected = False

    def getInfo(self):
        """ Return some information about the device manager.

        This returns in order:
        * the port name;
        * the model of the arduino board connected;
        * the name of the sketch uploaded on the board at the time of the last
        handshaking;
        * the version of the sketch uploaded on the board at the time of the
        last handshaking.
        """
        return (self.Port, self.Model, self.SketchName, self.SketchVersion)

    def handshake(self, targetSketchName = None, timeout = 5.0):
        """ Read the sketch information from the serial connection.

        I suspect we have some timing issue, here. Sometimes right after arduino
        has been plugged to the USB port the string being read during the
        handshaking is rubbish (i.e., 'thethermistor,1' instead of
        'thermistor,1').

        The flushInput() below *seems* to help but I'm not really sure about it.

        Note that we set the timeout to a long(ish) value as the handshaking
        takes some time. Therefore we store the connection timeout at the
        beginning of the method and restore it at the end.
        """
        if not self.connected():
            logger.error('Device manager not connected, cannot handshake.')
            return
        prevTimeout = self.timeout
        self.setTimeout(timeout)
        logger.info('Trying hand-shaking...')
        self.flushInput()
        info = self.readline()
        logger.debug('Device response: %s' % repr(info))
        try:
            (self.SketchName, self.SketchVersion) = info.split(',')
            self.SketchName = self.SketchName.strip('\x00')
            logger.info('Sketch "%s" version %s preloaded on the board.' %\
                            (self.SketchName, self.SketchVersion))
        except:
            logger.warn('Could not determine the sketch preloaded.')
        if targetSketchName is not None:
            targetSketchVersion = getSketchVersion(targetSketchName)
            if self.SketchName == targetSketchName and \
                    self.SketchVersion == targetSketchVersion:
                logger.info('It looks like the sketch is right, ready to go!')
                status = 0
            else:
                logger.info('Sketch "%s" version %s needed, instead.' %\
                                (targetSketchName, targetSketchVersion))
                status = 1
        else:
            logger.info('No target sketch defined.')
            status = 0
        self.setTimeout(prevTimeout)
        return status

    def readline(self, stripeol = True):
        """ Read a line from the serial connection. 
        
        Note that the eol is necessary for pyserial to tell that the line
        is finished.

        By default the eol character is stripped from the line.
        """
        line = serial.Serial.readline(self, None)
        if stripeol:
            line = line.strip(EOL_CHAR)
        return line
    
    def read(self, size = 1):
        """
        """
        return serial.Serial.read(self, size)

    def writeOne(self):
        """ Write a 1 to the serial connection.
        """
        self.write(self.ONE)

    def writeZero(self):
        """ Write a 0 to the serial connection.
        """
        self.write(self.ZERO)

    def setTimeout(self, timeout):
        """ Set a read timeout value.

        Possible values for the parameter timeout:
        timeout = None: wait forever
        timeout = 0: non-blocking mode (return immediately on read)
        timeout = x: set timeout to x seconds (float allowed)
        """
        if not self.connected():
            logger.error('Device manager not connected, cannot set timeout.')
            return
        if timeout is None:
            logger.info('Setting serial connection timeout to blocking...')
        else:
            logger.info('Setting serial connection timeout to %.2f s...' %\
                            timeout)
        self.timeout = timeout

    def setNonBlockingRead(self):
        """ Set non blocking read mode.
        """
        self.setTimeout(0)

    def readTimestamp(self):
        """ Read a timestamp from the serial port.

        Watch out: return None upon timeout.
        """
        payload = self.read(gTimestamp.LENGTH)
        if not payload:
            return None
        return gTimestamp(payload)

    def readAnalogReadout(self):
        """ Read an analog readout from the serial port.

        Watch out: return None upon timeout.
        """
        payload = self.read(gAnalogReadout.LENGTH)
        if not payload:
            return None
        return gAnalogReadout(payload)

    def loadSketch(self, sketchName):
        """ Load a sketch to arduino.

        This actually involves quote a few steps:
        (*) close the serial connection;
        (*) instantiate a gSketchLoader object and upload the sketch;
        (*) re-open the serial connection;
        (*) do a hand-shaking with the new sketch.
        """
        from plasduino.arduino.gSketchLoader import gSketchLoader
        self.close()
        loader = gSketchLoader(self.Port, self.Model)
        loader.upload(sketchName)
        self.open()
        return self.handshake(sketchName)

    def __str__(self):
        """ String formatting.
        """
        text = 'Arduino %s (vid %s, pid %s) connected on %s (baudrate = %s).' %\
            (self.Model, self.VID, self.PID, self.Port, BAUD_RATE)
        text += '\nSketch uploaded at last handshaking: "%s" version %s' %\
            (self.SketchName, self.SketchVersion)
        return text



if __name__ == '__main__':
    m = gArduinoManager()
    print(m)
