#!/usr/bin/env python
# *********************************************************************
# * Copyright (C) 2012--2013 Luca Baldini (luca.baldini@pi.infn.it)   *
# *                                                                   *
# * For the license terms see the file LICENCE, distributed           *
# * along with this software.                                         *
# *********************************************************************
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import time
import os

from PyQt4 import QtCore
from plasduino.daq.gRunControl import gRunControl
from plasduino.daq.gArduinoManager import gArduinoManager
from plasduino.__logging__ import logger


class gArduinoRunControl(gRunControl):

    """ Specialized RunControl class for the application using the arduino
    board.
    """

    SKETCH_NAME = None

    def __init__(self, eventBuffer, maxSeconds = None):
        """ Constructor.

        In addition to the features of the gRunControl event class we provide
        here all the facilities to exchange information with the arduino
        board.

        The additional signals emitted by this derived class are:
        (*) fatalError(QString) when problems are encountered while trying
        to establish communication with the board, the parameter being the
        text of the error message;
        (*) connectedToArduino() upon successfull connection to the board.
        """
        gRunControl.__init__(self, eventBuffer, maxSeconds)
        self.ArduinoManager = None

    def connectToArduino(self, timeout = None, autoUpload = True):
        """ Connect to the arduino board.
        
        The connection to the arduino board is deferred to a separate
        class method for two reasons:
        (i) this paradigm provides a neat separation between the applications
        that do require an arduino board and those who do not (e.g., the dice
        module or the audio applications);
        (ii) for the main window to intercept the signals we need to call
        the connectRunControl() method of the main window after the RunControl
        has been instantiated and before we attempt to connect to arduino.
        """
        self.ArduinoManager = gArduinoManager()
        if not self.ArduinoManager.connected():
            msg = 'The device manager doesn\'t seem to be connected any ' \
                'serial device, while at this time it really should.\n\n'
            log = self.ArduinoManager.getSerialLog()
            if log is not None:
                msg += 'The last (hopefully helpful) message from the device ' \
                    'manager was: "%s"' % log
                if log == gArduinoManager.COULD_NOT_AUTODETECT_USB_ERR:
                    msg += ' This might indicate that you don\'t have an ' \
                        'arduino board plugged in or you need to manually ' \
                        'configure the connection through the configuration ' \
                        'editor.'
                elif '[Errno 16]' in log:
                    msg += ' This might just indicate that you need to wait ' \
                        'a little bit for the communication to be ' \
                        'established (on old machines it might take as long ' \
                        'as 30+ s from the time you plug arduino). Do not ' \
                        'disconnect the board and try launching the ' \
                        'application in a little while.'
                elif '[Errno 13]' in log and os.name == 'posix':
                    msg += ' This might indicate that you need to add ' \
                        'yourself (as a GNU/Linux user) to the dialout group.'
            logger.error(msg.replace('\n\n', ' '))
            self.emit(QtCore.SIGNAL('fatalError(QString)'), msg)
        else:
            if timeout is not None:
                self.ArduinoManager.setTimeout(timeout)
            if self.ArduinoManager.handshake(self.SKETCH_NAME):
                if autoUpload:
                    if self.ArduinoManager.loadSketch(self.SKETCH_NAME):
                        abort('Giving up on the sketch, sorry')
            self.emit(QtCore.SIGNAL('connectedToArduino()'))

    def _start(self):
        """ Start the RunControl (STOPPED -> RUNNING).
        """
        gRunControl._start(self)
        self.ArduinoManager.writeOne()

    def _pause(self):
        """ Pause the RunController (RUNNING -> PAUSED).
        """
        gRunControl._pause(self)
        self.ArduinoManager.writeZero()

    def _resume(self):
        """ Resume the run controller (PAUSED -> RUNNING).
        """
        gRunControl._resume(self)
        self.ArduinoManager.writeOne()

    def _stop(self):
        """ Stop the RunControl (RUNNING -> STOPPED).
        """
        self.ArduinoManager.writeZero()
        self.flushInput()
        gRunControl._stop(self)

    def flushInput(self):
        """ Make sure all the input data wating to be transferred through the
        serial connection have been actually processed.

        This is meant to account for the events that might possibly be collected
        on the arduino in the finite time it takes for the writeZero() signal
        to be tranferred and detected by the microcontroller. At low rate the
        chance of this happening are fairly low, but already at ~500 Hz input
        rate this happens from time to time. We want to be sure all the data
        are collected---oterwise they might end up in the following run if the
        data acquisition is restarted.

        This is not very elegant, as we just sleep for a couple hundred ms
        in order to wait for arduino to push out the last stuff, if any.
        """
        logger.info('Flushing input data stream...')
        time.sleep(0.2)
        n0 = self.getNumEvents()
        while 1:
            n = self.getNumEvents()
            self.poll()
            if self.getNumEvents() == n:
                break
        dn = n - n0
        if dn:
            logger.info('Done, %s more event(s) transferred.' % dn)
        else:
            logger.info('Done, no events were waiting to be transferred.')

    def poll_timestamps_alternating_edges(self):
        """ Typical polling cycle for the time measurements.
        
        Implementation of a polling cycle with arduino having one interrupt
        attached to both edges of a digital input.

        Here we essentially read the timestamps from the board and we ensure on
        the fly that each time measurement is on the opposite edge with
        respect to the previous one. If that's not the case the DAQ has really
        lost a transition and we do stop the run (as this would cause spurious
        data to appear in the processed data stream).
        """
        timestamp = self.ArduinoManager.readTimestamp()
        if timestamp is not None:
            previous = self.getLastEvent()
            if self.getNumEvents() and timestamp['edge'] == previous['edge']:
                logger.error('%s detected 2 consecutive timestamps on the '
                             'same edge.' % self.__class__.__name__)
                logger.info('(%d) %s' % (self.getNumEvents(), timestamp))
                logger.info('(%d) %s' % (self.getNumEvents() - 1, previous))
                dt = timestamp['seconds'] - previous['seconds']
                logger.info('Delta time: %.6f' % dt)
                logger.error('At least one timestamp got lost, '
                             'forcing stop run...')
                self.setStopped()
            else:
                self.addEvent(timestamp)
