#!/usr/bin/env python
# *********************************************************************
# * Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)         *
# *                                                                   *
# * For the license terms see the file LICENCE, distributed           *
# * along with this software.                                         *
# *********************************************************************
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import time

from plasduino.__logging__ import logger


class gEventBuffer(list):

    """ Class describing an event buffer.

    This is essentially a list with some additional utilities (e.g., to dump
    the content into a file.)

    The class also provides a full() hook that can be used downstream for
    interrupting the data acquisition after a given number of events.
    Consumer applications should be use the full() class method in the event
    loop, in conjunction with the setMaxNumEvents() method.

    We put some thought about enforcing the maximum number of events by means
    of a full() signal to be emitted in the fill() method and to be intercepted
    by the downstream code. This had the drawback that by the time the signal
    is actually intercepted, more events (and in some applications, like the
    plasduino_dice module, *many* more events) could be acquired, so that
    the strategy was ineffective. It wasn't even any faster, as it still
    required one check in the inner event loop.

    Finally, note that if we want to go for a signal/slot paradigm, we need
    to make gEventBuffer inherit from QObject and, to avoid the conflicting
    metaclass issue, we can't inherit from list (i.e. we have to use
    composition rather than inheritance).
    """

    PROCESSED_DATA_FIELDS = []
    PROCESSED_DATA_UNITS = []
    MIN_NUM_EVENTS = 1

    def __init__(self, maxNumEvents = None):
        """ Constructor.
        """
        list.__init__(self)
        self.setMaxNumEvents(maxNumEvents)
        self.clear()

    def clear(self):
        """ Clear the buffer.
        """
        del self[:]

    def full(self):
        """ Return True if the buffer is full.
        """
        return len(self) >= self.__MaxNumEvents

    def setMaxNumEvents(self, numEvents = None):
        """ Set the maximum number of events to be acquired.

        If None is passed as an argument, the self.__MaxNumEvents class member
        is set to a very large number. This is to avoid a second condition
        to be checked in the full() class method, which is called in the inner
        loop and therefore should be as fast as possible.
        """
        if numEvents is None:
            numEvents = 1000000000000000
        logger.info('Setting the data buffer maximum size to %d.' % numEvents)
        self.__MaxNumEvents = numEvents

    def fill(self, event):
        """ Append an event to the buffer.

        Note that, for minimizing any overhead, we don't perform any check on
        self.__MaxNumEvents, here. It is responsibility of consumer applications
        to prevent the data buffer to grow beyond limits.
        """
        self.append(event)

    def write(self, filePath):
        """ Write the buffer to file in binary format.

        Under normal condition this dumps into a file what has been read out
        from arduino through the serial interface.
        """
        logger.info('Writing raw data to binary file %s...' % filePath)
        outputFile = open(filePath, 'wb')
        for event in self:
            outputFile.writelines(event.Payload)
        outputFile.close()
        logger.info('Done, %d event(s) written.' % len(self))

    def getFileHeader(self):
        """ Utility method to format the header for the processed text file.

        We do write the name of the module producing the file, a timestamp
        and the field names (possibly with units), as consecutive lines
        beginning with a # (so that gnuplot will happily ignore them).
        """
        timestamp = time.asctime()
        header =  '#\n'
        header += '# Written by plasduino.%s on %s.\n' %\
            (self.__class__.__name__, timestamp)
        if len(self.PROCESSED_DATA_FIELDS):
            header += '# '
            for field in self.PROCESSED_DATA_FIELDS:
                header += '%s\t\t' % field
            header = '%s\n' % header.rstrip('\t')
        if len(self.PROCESSED_DATA_UNITS):
            header += '# '
            for unit in self.PROCESSED_DATA_UNITS:
                header += '[%s]\t\t' % unit
            header = '%s\n' % header.rstrip('\t')
        header += '#\n'
        return header

    def process(self, filePath):
        """ Process the event buffer and write a 'baked' output file.

        The base class method is a do-nothing one. The derived classes have the
        responsibility to implement the desired behavior.
        """
        pass



if __name__ == '__main__':
    b = gEventBuffer()
    b.fill(3)
    b.fill(973256)
    print(b)
    b.clear()
    print(b)
    b.fill(32)
    print(b)
