#!/usr/bin/env python
# *********************************************************************
# * Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)         *
# *                                                                   *
# * For the license terms see the file LICENCE, distributed           *
# * along with this software.                                         *
# *********************************************************************
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


from plasduino.daq.gEvent import gEvent
from plasduino.__logging__ import logger
from plasduino.arduino.__arduino__ import TIMESTAMP_HEADER


class gTimestamp(gEvent):

    """ An arduino timestamp is a 5-bit binary array:
    * byte(s) 0  : a header;
    * byte(s) 1-4: the actual time in us from micros().

    The corresponding write function on the arduino side is defined
    in arduino/plasduino.h and arduino/plasduino.c.
    """

    LENGTH = 5
    LAYOUT = [('header' , 'B', 0),
              ('seconds', 'L', 1)
              ]
    RISING_EDGE = 1
    FALLING_EDGE = 0

    def __init__(self, payload):
        """ Constructor.

        Notice that we convert ms to s and we check the header.
        """
        gEvent.__init__(self, payload)
        self['seconds'] *= 1e-6
        header = self['header']
        if header & 0xF0 != TIMESTAMP_HEADER:
            logger.error('Malformed timestamp (payload = %s).' %\
                             (repr(self.Payload)))
        self['pinNumber'] = header & 0x7
        self['edge'] = (header >> 3) & 0x1

    def rising(self):
        """ Returns whether the timestamp corresponds to a transition on
        the rising egde.
        """
        return self['edge'] == self.RISING_EDGE

    def falling(self):
        """ Returns whether the timestamp corresponds to a transition on
        the falling egde.
        """
        return self['edge'] == self.FALLING_EDGE

    def __str__(self):
        """ String representation.
        """
        return '%s -> %.6f s (header = 0x%x, pin = %d, edge = %d)' %\
            (repr(self.Payload), self['seconds'], self['header'],
             self['pinNumber'], self['edge'])



if __name__ == '__main__':
    t0 = gTimestamp('\xa3\x00,O\xe0')
    print(t0)
    t1 = gTimestamp('\xab\x00,O\xe0')
    print(t1)
    t2 = gTimestamp('\xbb\x00,O\xe0')
    print(t2)
