#!/usr/bin/env python
# *********************************************************************
# * Copyright (C) 2012--2013 Luca Baldini (luca.baldini@pi.infn.it)   *
# *                                                                   *
# * For the license terms see the file LICENCE, distributed           *
# * along with this software.                                         *
# *********************************************************************
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import PyQt4.Qwt5 as Qwt

from PyQt4 import QtCore, QtGui, Qt
from plasduino.gui.gAcquisitionWindow import gAcquisitionWindow
from plasduino.gui.gPlotWidget import gPlotWidget
from plasduino.gui.gTransportBar import gTransportBar
from plasduino.gui.gAcquisitionDataDisplay import gAcquisitionDataDisplay
from plasduino.gui.gQt import connect
from plasduino.gui.gStripChart import gStripChart
from plasduino.__logging__ import logger


class gAnalogMonitorWindow(gAcquisitionWindow):
    
    """
    """

    WINDOW_TITLE = 'Analog monitor'
    COLORS = [Qt.Qt.red, Qt.Qt.blue]

    def __init__(self, **kwargs):
        """ Constructor.

        Since, depending on the context, it's handy to be able to address the
        plots on the canvas both by name and by insertion order, we keep
        both a dictionary and a list of the active plots.
        """
        gAcquisitionWindow.__init__(self, **kwargs)
        self.StripChartDict = {}
        self.StripChartList = []
        self.Zoomer = Qwt.QwtPlotZoomer(Qwt.QwtPlot.xBottom,
                                        Qwt.QwtPlot.yLeft,
                                        Qwt.QwtPicker.DragSelection,
                                        Qwt.QwtPicker.AlwaysOff,
                                        self.PlotWidget.canvas())
        self.Zoomer.setRubberBandPen(Qt.QPen(Qt.Qt.black))
        self.disableZooming()

    def setup(self):
        """ Setup the widgets on the gui.
        """
        self.PlotWidget = gPlotWidget(self, xTitle = 'Time [s]')
        self.addWidget(self.PlotWidget)
        self.DataDisplay = gAcquisitionDataDisplay(self)
        self.addWidget(self.DataDisplay)
        self.DataDisplay.setMinimumWidth(self.PlotWidget.width())
        self.TransportBar = gTransportBar(self, tooltips = \
                                              gAcquisitionWindow.TOOLTIP_DICT) 
        self.addWidget(self.TransportBar)
        self.TransportBar.setMinimumWidth(self.PlotWidget.width())
        connect(self.TransportBar, 'quit()', self.close)

    def enableZooming(self):
        """ Enable zooming.
        """
        self.Zoomer.setEnabled(True)
        self.Zoomer.setZoomBase()

    def disableZooming(self):
        """ Disable zooming.
        """
        self.Zoomer.setEnabled(False)

    def addPlot(self, name, **kwargs):
        """ Add a plot on the main widget.

        Note that the plot gets automatically indexed by its name in the
        StripChartDict class member and appended to the StripChartList class
        member.

        We also take care of connecting the legend items to the plot
        toggle() slot, here.
        """
        chart = gStripChart(name, **kwargs)
        self.StripChartDict[name] = chart
        self.StripChartList.append(chart)
        chart.attach(self.PlotWidget)
        for item in self.PlotWidget.Legend.legendItems():
            if item.text().text() == name:
                connect(item, 'clicked()', chart.toggle)

    def getPlotByName(self, name):
        """ Retrieve a specific plot by name.
        """
        return self.StripChartDict[name]

    def getPlotByIndex(self, index):
        """ Retrieve a specific plot by index (i.e. insertion order).
        """
        return self.StripChartList[index]

    def updatePlots(self):
        """ Do-nothing method to be reimplemented in derived classes.
        """
        pass

    def clearPlots(self):
        """ Clear all the plots on the main plot widget.
        """
        logger.info('Cleaning up strip charts...')
        for chart in self.StripChartDict.values():
            chart.clear()



if __name__ == '__main__':
    from plasduino.__cfgparse__ import getapp
    application = getapp()
    w = gAnalogMonitorWindow()
    w.show()
    application.exec_() 
