#!/usr/bin/env python
# *********************************************************************
# * Copyright (C) 2012--2103 Luca Baldini (luca.baldini@pi.infn.it)   *
# *                                                                   *
# * For the license terms see the file LICENCE, distributed           *
# * along with this software.                                         *
# *********************************************************************
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import re

from PyQt4 import QtCore, QtGui

from plasduino.gui.gQt import connect



class gFileType:

    """ Small utility class representing a file type.
    """
    
    def __init__(self, extension, description):
        """ Constructor.
        """
        self.Extension = extension
        self.Description = description

    def getFilter(self):
        """ Return the filter to be used in a file dialog.
        """
        return '%s (*.%s)' % (self.Description, self.Extension)


""" The Qt separator for multiple file types in a dialog.
"""
FILTER_SEPARATOR = ';;'

def getFilter(fileTypes):
    """ Return the filter for a list of file types.
    """
    filterExpr = ''
    for fileType in fileTypes:
        filterExpr += '%s;;' % fileType.getFilter()
    return filterExpr.rstrip(';')


""" Define some useful file types.
"""
FILE_TYPE_TXT = gFileType('txt', 'Plain text')
FILE_TYPE_CSV = gFileType('csv', 'Comma-separated values')
FILE_TYPE_DAT = gFileType('dat', 'plasduino binary data')



class gFileDialog(QtGui.QFileDialog):

    """ Small wrapper class around a QFileDialog object.
    """

    pass



class gFileOpenDialog(QtGui.QFileDialog):

    """ Small wrapper class around a QFileDialog object.
    """

    def __init__(self, parent = None, caption = '', directory = '',
                 filterExpr = ''):
        """ Constructor.
        """
        QtGui.QFileDialog.__init__(self, parent, caption, directory, filterExpr)
        self.setAcceptMode(QtGui.QFileDialog.AcceptOpen)



class gFileSaveDialog(QtGui.QFileDialog):

    """ Small wrapper class around a QFileDialog object.

    Among the other things, here we prevent the user from inserting spaces in
    the file names (sorry for the Windows users).
    """

    def __init__(self, fileTypes, parent = None, caption = '', directory = ''):
        """ Constructor.
        """
        filterExpr = getFilter(fileTypes)
        QtGui.QFileDialog.__init__(self, parent, caption, directory, filterExpr)
        self.setAcceptMode(QtGui.QFileDialog.AcceptSave)
        self.LineEdit = self.findChild(QtGui.QLineEdit)
        connect(self.LineEdit, 'textChanged(QString)', self.checkFileName)

    def checkFileName(self, text):
        """ Make sure the user does not put any space in the file name.
        """
        if ' ' in text:
            self.LineEdit.setText(text.replace(' ', ''))



def getSaveFilePath(fileTypes, parent = None, caption = '', directory = ''):
    """ Convenience function for retrieving a save file path.

    Note that we enforce here that the file name ends with the file
    extension.
    """
    dialog = gFileSaveDialog(fileTypes, parent, caption, directory)
    if dialog.exec_():
        filePath = str(dialog.selectedFiles()[0])
        selectedFilter = dialog.selectedNameFilter()
        regexp = '(?<=\(\*.)[a-zA-Z0-9]*(?=\))'
        extension = re.search(regexp, selectedFilter).group(0)
        if not filePath.endswith('.%s' % extension):
            filePath = '%s.%s' % (filePath, extension)
        return filePath
    else:
        return None



class gFileDialogWidget(QtGui.QLineEdit):

    """ A line edit prompting a file dialog when you click on.
    """

    def __init__(self, parent = None, text = ''):
        """
        """
        QtGui.QLineEdit.__init__(self, text, parent)
        
    def mouseReleaseEvent(self, e):
        """
        """
        fileDialog = gFileDialog(self)
        if fileDialog.exec_():
            self.setText(fileDialog.selectedFiles()[0])



class gFolderDialogWidget(gFileDialogWidget):
    
    """ A line edit prompting a folder dialog when you click on.
    """

   
    def mouseReleaseEvent(self, e):
        """
        """
        fileDialog = gFileDialog(self)
        fileDialog.setFileMode(QtGui.QFileDialog.Directory)
        if fileDialog.exec_():
            self.setText(fileDialog.selectedFiles()[0])


if __name__ == '__main__':
    from plasduino.__cfgparse__ import getapp
    application = getapp()
    print(getSaveFileName([FILE_TYPE_TXT, FILE_TYPE_CSV]))
