#!/usr/bin/env python
# *********************************************************************
# * Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)         *
# *                                                                   *
# * For the license terms see the file LICENCE, distributed           *
# * along with this software.                                         *
# *********************************************************************
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


""" Release manager for the plasduino package.
"""

import os
import sys
import time

import plasduino.__utils__ as __utils__

from plasduino.__logging__ import logger, abort
from plasduino.__plasduino__ import PLASDUINO_DOC, PLASDUINO_DEBIAN


TAG_FILE_NAME = '__tag__.py'
BUILD_DIR = 'build'
DIST_DIR = 'dist'

RELEASE_NOTES_PATH = os.path.join(PLASDUINO_DOC, 'release.notes')
DEBIAN_CHANGELOG_PATH = os.path.join(PLASDUINO_DEBIAN, 'changelog')

""" The build date should be compliant to the debian standards, as documented at
http://www.debian.org/doc/debian-policy/ch-source.html
"""
BUILD_DATE = time.strftime('%a, %d %b %Y %H:%M:%S %z')

DEBIAN_PACKAGER = 'Jacopo Nespolo <j.nespolo@gmail.com>'

TAG_MODES = ['major', 'minor', 'patch']


def updateVersionFile(mode, dryRun = False):
    """ Update the __tag__.py module with the new tag and build date.
    """
    from plasduino.__tag__ import TAG as PREV_TAG
    logger.info('Previous tag was %s...' % PREV_TAG)
    version, release, patch = [int(item) for item in PREV_TAG.split('.')]
    if mode == 'major':
        version += 1
        release = 0
        patch = 0
    elif mode == 'minor':
        release += 1
        patch = 0
    elif mode == 'patch':
        patch += 1
    else:
        abort('Unknown release mode %s.' % mode)
    nextTag = '%s.%s.%s' % (version, release, patch)
    logger.info('Writing new tag (%s) to %s...' % (nextTag, TAG_FILE_NAME))
    if not dryRun:
        outputFile = open(TAG_FILE_NAME, 'w')
        outputFile.writelines('TAG = \'%s\'\n' % nextTag)
        outputFile.writelines('BUILD_DATE = \'%s\'\n' % BUILD_DATE)
        outputFile.close()
    logger.info('Done.')
    return nextTag

def updateReleaseNotes(tag, dryRun = False):
    """ Write the new tag and build date on top of the release notes
    (which must be kept up to date during the release process).
    """
    hline = '-'*79
    logger.info('Reading in %s...' % RELEASE_NOTES_PATH)
    notes = open(RELEASE_NOTES_PATH).read().strip('\n')
    logger.info('Writing out %s...' % RELEASE_NOTES_PATH)
    if not dryRun:
        outputFile = open(RELEASE_NOTES_PATH, 'w')
        outputFile.writelines('\n%s\nplasduino (%s) - %s\n%s\n' %\
                                  (hline, tag, BUILD_DATE, hline))
        outputFile.writelines(notes)
        outputFile.close()
    logger.info('Done.')

def debianizeReleaseNotes(dryRun = False):
    """ Process the release notes file and produce a new one conforming to the
    debian rules.
    """
    logger.info('Processing %s and writing out %s...' %\
                    (RELEASE_NOTES_PATH, DEBIAN_CHANGELOG_PATH))
    inputFile = open(RELEASE_NOTES_PATH)
    outputFile = open(DEBIAN_CHANGELOG_PATH, 'w')
    for line in inputFile:
        if line.startswith('-----'):
            pass
        elif line.startswith('plasduino'):
            tag, date = line.split(' - ')
            notes = []
            line = inputFile.next()
            assert line.startswith('-----')
            line = inputFile.next()
            while len(line.strip('\n').strip()):
                notes.append(line)
                try:
                    line = inputFile.next()
                except StopIteration:
                    break
            outputFile.writelines('%s UNRELEASED; urgency=low\n\n' % tag)
            for note in notes:
                outputFile.writelines(note)
            outputFile.writelines('\n -- %s  %s\n' % (DEBIAN_PACKAGER, date))
    outputFile.close()
    inputFile.close()
    logger.info('Done.')

def tagPackage(mode, dryRun = False):
    """ Tag the package.

    This means:
    (*) hg pull/update to make sure we're not missing remote modification;
    (*) figure out the target tag, update the release.notes and create the
    debian version;
    (*) commit the modifications, tag and push.
    """
    __utils__.cmd('hg pull', verbose = True, dryRun = dryRun)
    __utils__.cmd('hg update', verbose = True, dryRun = dryRun)
    __utils__.cmd('hg status', verbose = True, dryRun = dryRun)
    tag = updateVersionFile(mode, dryRun)
    updateReleaseNotes(tag, dryRun)
    debianizeReleaseNotes(dryRun)
    msg = 'Prepare for tag %s.' % tag
    __utils__.cmd('hg commit -m "%s"' % msg, verbose = True, dryRun = dryRun)
    __utils__.cmd('hg push', verbose = True, dryRun = dryRun)
    __utils__.cmd('hg tag %s' % tag, verbose = True, dryRun = dryRun)
    __utils__.cmd('hg push', verbose = True, dryRun = dryRun)
    __utils__.cmd('hg status', verbose = True, dryRun = dryRun)
    return tag

def dist(tag = None):
    """ Release plasduino.
    """
    if tag is None:
        from plasduino.__tag__ import TAG as tag
    # Create a clean src distr
    __utils__.cmd('python setup.py sdist --dist-dir=%s --prune' % DIST_DIR,
                  verbose = False, logFilePath = 'src.log')
    srcArcPath = os.path.join(DIST_DIR, 'plasduino-%s.tar.gz' % tag)
    if not os.path.exists(srcArcPath):
        abort('Oops... why there is no %s?' % srcArcPath)
    # Prepare the relevant paths...
    srcFolderPath = os.path.join(DIST_DIR, 'plasduino-%s' % tag)
    rpmFolderPath = '%s.rpm' % srcFolderPath
    debFolderPath = '%s.deb' % srcFolderPath
    # Create the rpm package...
    logger.info('Creating rpm package...')
    prog = 'rpmbuild'
    if not __utils__.programInstalled(prog)[0]:
        logger.warn('%s is not installed, skipping rpm dist...' % prog)
    else:
        __utils__.cmd('tar xzf %s --directory %s' % (srcArcPath, DIST_DIR),
                      verbose = False)
        __utils__.mv(srcFolderPath, rpmFolderPath)
        __utils__.rmdir(os.path.join(rpmFolderPath, 'debian'))
        __utils__.cp('post_install', rpmFolderPath)
        __utils__.cmd('cd %s; python setup.py bdist_rpm --dist-dir=..' %\
                          rpmFolderPath, verbose = False,
                      logFilePath = 'rpm.log')
        __utils__.rmdir(rpmFolderPath)
    # Create the debian package...
    logger.info('Creating debian package...')
    prog = 'dpkg-buildpackage'
    if not __utils__.programInstalled(prog)[0]:
        logger.warn('%s is not installed, skipping deb dist...' % prog)
    else:
        __utils__.cmd('tar xzf %s --directory %s' % (srcArcPath, DIST_DIR),
                      verbose = False)
        __utils__.mv(srcFolderPath, debFolderPath)
        __utils__.rm(os.path.join(debFolderPath, 'LICENSE'))
        __utils__.cmd('cd %s; dpkg-buildpackage -i -I -rfakeroot' %\
                          debFolderPath, verbose = False,
                      logFilePath = 'deb.log')
        __utils__.rmdir(debFolderPath)

def dumpModuleInfo():
    """ Run the code dumping the module info, faking the post-installation
    script.
    
    This is just to make sure that the post-installation script won't
    crash at install time.
    """
    logger.info('Dumping module info (making sure the post-installation '
                'script won\'t crash).')
    from plasduino.modulemanager.__modulemanager__ import dumpModuleInfo
    from plasduino.__plasduino__ import PLASDUINO_DIST_MODULEINFO_FILE_PATH,\
        PLASDUINO_MODULES
    dumpModuleInfo([PLASDUINO_MODULES], PLASDUINO_DIST_MODULEINFO_FILE_PATH)



if __name__ == '__main__':
    dumpModuleInfo()
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option('-t', dest = 'tagmode', type = str, default = None,
                      help = 'The release tag mode %s.' % TAG_MODES)
    parser.add_option('-n', action = 'store_true', dest = 'dryrun',
                      help = 'Dry run (i.e. do not actually do anything).')
    parser.add_option('-d', action = 'store_true', dest = 'dist',
                      help = 'Create distribution files.')
    (opts, args) = parser.parse_args()
    if not opts.tagmode and not opts.dist:
        parser.error('Please specify at least one among the -t and -d options.')
    tag = None
    if opts.tagmode is not None:
        if opts.tagmode not in TAG_MODES:
            parser.error('Invalid tag mode %s (allowed: %s)' %\
                             (opts.tagmode, TAG_MODES))
        tag = tagPackage(opts.tagmode, opts.dryrun)
    if opts.dist and not opts.dryrun:
        dist(tag)
