#!/usr/bin/env python
# *********************************************************************
# * Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)         *
# *                                                                   *
# * For the license terms see the file LICENCE, distributed           *
# * along with this software.                                         *
# *********************************************************************
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import math

from plasduino.sensors.gThermistor import gThermistor


class NXFT15XH103FA2B(gThermistor):

    """ The NXFT15XH103FA2B thermistor.
    """

    def __init__(self):
        """ Constructor.

        The coefficients for the Steinhart-Hart equation are based on the
        output of the fitData() function below.
        """
        gThermistor.__init__(self, 'NXFT15XH103FA2B', 'MURATA MANUFACTURING',
                             8.65082e-04, 2.55459e-04, 1.74569e-07)

    def fitData(self, filePath = 'NXFT15XH103FA2B.dat'):
        """ Read the manufacturer data and find the coefficients for the
        Steinhart-Hart equation.
        """
        import sys
        import os
        sys.path.append(os.path.join('..', 'doc', 'macros'))
        from __root__ import ROOT, TCanvas, store
        gAbs = ROOT.TGraph()
        store(gAbs)
        gAbs.SetMarkerStyle(24)
        dataT = []
        dataR = []
        for line in open(filePath):
            if not line.startswith('#'):
                line = line.strip()
                T, Rmin, R, RMax = [float(item) for item in line.split()]
                T += 273.15
                R *= 1000.
                dataT.append(T)
                dataR.append(R)
                gAbs.SetPoint(gAbs.GetN(), R, T)
        cAbs = TCanvas('NXFT15XH103FA2B calibration', grids = True, logx = True)
        gAbs.GetXaxis().SetTitle('R [#Omega]')
        gAbs.GetYaxis().SetTitle('T [#circ K]')
        gAbs.Draw('ap')
        f = ROOT.TF1('f', '1./([0] +[1]*log(x) +[2]*(log(x)**3))')
        f.SetParameter(0, 1e-3)
        f.SetParameter(1, 1e-4)
        f.SetParameter(2, 1e-7)
        store(f)
        gAbs.Fit('f')
        cAbs.Update()
        gRes = ROOT.TGraph()
        gRes.SetMarkerStyle(24)
        store(gRes)
        for T, R in zip(dataT, dataR):
            gRes.SetPoint(gRes.GetN(), R, f.Eval(R) - T)
        cRes = TCanvas('NXFT15XH103FA2B residuals', grids = True, logx = True)
        gRes.GetXaxis().SetTitle('R [#Omega]')
        gRes.GetYaxis().SetTitle('Fit residuals [#circ K]')
        gRes.Draw('ap')
        cRes.Update()
        


if __name__ == '__main__':
    thermistor = NXFT15XH103FA2B()
    #thermistor.fitData()
    print(thermistor)
